
#include <stdio.h>
#include <stdarg.h>

//#include <Windows.h>
#include <GL/gl.h>

#include "DrawText.h"
#include <assert.h>

#include <string.h>

#include "tga.h"

// Util functions.  Global as they may come in useful elsewhere.

// Switches to 2d ortho projection mode.
// Use in conjunction with Exit2d()
void Go2d()
{

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();									// Store The Projection Matrix
	glLoadIdentity();								// Reset The Projection Matrix
	glOrtho(0,1024,0,768,-1,1);						// Set Up An Ortho Screen
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();									// Store old Modelview Matrix
	glLoadIdentity();								// Reset The Modelview Matrix
}


// Switches out of 2d ortho projection mode.
// Use in conjunction with Go2d()
void Exit2d()
{
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();									// Restore old Projection Matrix
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();									// Restore old Projection Matrix
}


// The single instance of our class.
CTextDrawer* CTextDrawer::mSingleton = NULL;


// Provides access to the single global CTextDrawer object.
CTextDrawer &CTextDrawer::GetSingleton()
{
	if (!mSingleton)
	{
		mSingleton = new CTextDrawer;
		Initialise();
	}

	return *mSingleton;
}


// Call when finished.
void CTextDrawer::Destroy()
{
	if (mSingleton)
	{
		// Delete the gl display lists for each letter.
		glDeleteLists(mSingleton->mBaseListId, NUMCHARS);			// Delete All Display Lists

                glDeleteTextures(1,&(mSingleton->mFontTexId));
		delete mSingleton;
		mSingleton = NULL;
	}
}


CTextDrawer::CTextDrawer()
{
	mBaseListId = 0;
	mR = mG = mB = mA = 1.0f;
}


// Sets the font texture GL texture id.
// Param: aFontTextureId - A GL texture enum for the font texture.
void CTextDrawer::SetFontTextureId(GLuint aFontTextureId)
{
	mSingleton->mFontTexId = aFontTextureId;
}


void CTextDrawer::Initialise()		// Build Our Font Display List
{
	assert(mSingleton->mFontTexId);		// No font texture id set: Use TEXTDRAWER.SetFontTextureId(aFontTextureId);

	float	cx;										// Holds Our X Character Coord
	float	cy;										// Holds Our Y Character Coord
	int		loop;

        mSingleton->mFontTexId = createTexture("font_arial.tga");

	mSingleton->mBaseListId = glGenLists(NUMCHARS);				// Creating NUMCHARS Display Lists
	glBindTexture(GL_TEXTURE_2D, mSingleton->mFontTexId);
	for (loop=0; loop<NUMCHARS; loop++)
	{
         // alexander kramer :  sligthly modified
          
		cx=((float)(loop%16))/16.0f;				// X Position Of Current Character
		cy=(((float)(loop/16))/16.0f);				// Y Position Of Current Character

		glNewList(mSingleton->mBaseListId+loop, GL_COMPILE);	// Start Building A List
			glBegin(GL_QUADS);						// Use A Quad For Each Character
				glTexCoord2f(cx,cy);		// Texture Coord (Bottom Left)
				glVertex2i(0,0);					// Vertex Coord (Bottom Left)
				glTexCoord2f(cx+0.0625f,cy);// Texture Coord (Bottom Right)
				glVertex2i(16,0);					// Vertex Coord (Bottom Right)
				glTexCoord2f(cx+0.0625f,cy+0.0625f);		// Texture Coord (Top Right)
				glVertex2i(16,16);					// Vertex Coord (Top Right)
				glTexCoord2f(cx,cy+0.0625f);				// Texture Coord (Top Left)
				glVertex2i(0,16);					// Vertex Coord (Top Left)
			glEnd();								// Done Building Our Quad (Character)
			glTranslatef(12.0f,0,0);				// Move To The Right Of The Character
		glEndList();								// Done Building The Display List
	}
}


// Sets the colour of the text.
void CTextDrawer::SetColor(float aR, float aG, float aB, float aA)
{
	mR = aR;
	mG = aG;
	mB = aB;
	mA = aA;
}


/*
INFO From siggraph:

The GL_INTENSITY texture format is a good texture format for textures containing glyphs
because the texture format is compact. The intensity texture format replicates the single
intensity component value in the red, green, blue, and alpha channels. When rendering
colored glyphs, use the GL_MODULATE texture environment and set the current color to the
desired glyph color.
*/

// alexander kramer : original code was changed by me  
bool CTextDrawer::PrintText(bool bold, int maxLength, const char *fmt, ...)
{
	assert(mBaseListId != 0);

	char		text[1024];							// Holds Our String
	va_list		ap;									// Pointer To List Of Arguments

	if (fmt == NULL)
		return true;

	// Parse The String For Variables
	va_start(ap, fmt);							
	vsprintf(text, fmt, ap);
	va_end(ap);

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, mFontTexId);
         //glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
       
	//glDisable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);								// For transparent background
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	//Go2d();

	//glTranslated(x, y, 0);							// Position The Text (0,0 - Top Left)

	// Note: the 128 below refers to the font texture containing 2 font sets.
	// Use 0 for the first set, or 128 for the second set.

        if (bold)
          glListBase(mBaseListId - ' ' + 0);			// Font bitmaps starts at ' ' (space/32).
        else
          glListBase(mBaseListId - ' ' + 128);			// Font bitmaps starts at ' ' (space/32).

	//glColor4f(mR, mG, mB, mA);			// Set text colour.


        if (maxLength == -1)
          glCallLists(strlen(text), GL_BYTE, text);		// Write The Text To The Screen
        else
          glCallLists(maxLength, GL_BYTE, text);		// Write The Text To The Screen


       	glDisable(GL_TEXTURE_2D);
	//Exit2d();

       	glDisable(GL_BLEND);
	//glEnable(GL_DEPTH_TEST);

	return true;
}

