// Class QCaptionButton 
// Filename QCaptionButton.h 
// FaPra 2003-2004, A5, Alexander Kramer

#ifndef QCaptionButton_h 
#define QCaptionButton_h 


//! Pre-declaration of QCaptionButton
class QCaptionButton;


#include "QButton.h"
#include "QFont.h"

//! Button with Caption
class QCaptionButton : public QButton
{
private:
//! x position of the caption (centered)
int CaptionPositionX;

protected:
//! caption of the button
string Caption;

//! used to draw caption
QFont* Font;

public:
//! overwrite this to recalculate caption position
virtual void updateCaptionPosition();

//! draws itself with OpenGL. Here is the OpenGL drawing code
virtual void draw( QEvent*  e );

//! layout the children widgets. This method is called every time the size is changed. Have to be overwritten for respond on size changes
virtual void layout();

//! have to return true if widget should be dragged with mouse. This method is overwritten in sub classes. If returns true, widget will be dragged
virtual bool canDrag( QEvent*  e );

//! processing mouse/keyboard events which are in widgets region. Mouse coordinates are relative to Owner
virtual bool processEvent( QEvent*  e );

//! processing mouse events which are in Owner widget region. Mouse coordinates are relative to Owner
virtual bool processMouseOwner( QEvent*  e );

//! called if a child is added/removed
virtual void childrenChanged();

//! defines whether control can have keyboard focus. Should return true if Widget wants to get keyboard focus
virtual bool canFocus( QEvent*  e );

//! initialization code
virtual void Init();

//! clean up code
virtual void CleanUp();

//! getter method for Caption
virtual string getCaption();

//! setter method for Caption
virtual void setCaption( string  newValue );

//! getter method for Font
 QFont* getFont();

//! prints itself to the cout
 void print();

//! Constructor
  QCaptionButton( QWidget*  aOwner );

//! Destructor
virtual  ~QCaptionButton();


};
#endif

