// Class QCheckBox 
// Check Box
// Filename QCheckBox.cpp
// FaPra 2003-2004, A5,  Alexander Kramer


#include "QCheckBox.h"
#include <iostream>
using namespace std;

// ----------------------------------------------------------

//! draws itself with OpenGL. Here is the OpenGL drawing code
void QCheckBox::draw( QEvent*  e )
{

 LookAndFeel->drawCheckBox(State == cbChecked);

 glTranslatef(BoxSize+2,(BoxSize-Font->getSize())/2,0);
 Font->drawInternal(Caption.c_str(),-1);

} // end of draw()

//! layout the children widgets. This method is called every time the size is changed. Have to be overwritten for respond on size changes
void QCheckBox::layout()
{
  if (Height < BoxSize) Height = BoxSize;
} // end of layout()

//! have to return true if widget should be dragged with mouse. This method is overwritten in sub classes. If returns true, widget will be dragged
bool QCheckBox::canDrag( QEvent*  e )
{
 return false;
} // end of canDrag()

//! processing mouse/keyboard events which are in widgets region. Mouse coordinates are relative to Owner
bool QCheckBox::processEvent( QEvent*  e )
{
 if (e->getType() == etMouseDown)
 {
   if (State == cbChecked) State = cbUnchecked;
   else State = cbChecked;
 }
 return false;
} // end of processEvent()

//! processing mouse events which are in Owner widget region. Mouse coordinates are relative to Owner
bool QCheckBox::processMouseOwner( QEvent*  e )
{
 return false;
} // end of processMouseOwner()

//! called if a child is added/removed
void QCheckBox::childrenChanged()
{
} // end of childrenChanged()

//! defines whether control can have keyboard focus. Should return true if Widget wants to get keyboard focus
bool QCheckBox::canFocus( QEvent*  e )
{
 return true;
} // end of canFocus()

//! initialization code
void QCheckBox::Init()
{
  Font->copyFrom(LookAndFeel->getFont());
  QWidget::Init();
} // end of Init()

//! clean up code
void QCheckBox::CleanUp()
{
} // end of CleanUp()

//! getter method for State
QCheckBoxState QCheckBox::getState()
{
return State;
} // end of getState()

//! setter method for State
void QCheckBox::setState( QCheckBoxState  newValue )
{
State = newValue;
} // end of setState()

//! getter method for BoxSize
int QCheckBox::getBoxSize()
{
return BoxSize;
} // end of getBoxSize()

//! setter method for BoxSize
void QCheckBox::setBoxSize( int  newValue )
{
BoxSize = newValue;
} // end of setBoxSize()

//! getter method for Caption
string QCheckBox::getCaption()
{
return Caption;
} // end of getCaption()

//! setter method for Caption
void QCheckBox::setCaption( string  newValue )
{
 Caption = newValue;
 setWidth((int)Font->getTextWidth(newValue.c_str())+BoxSize+2);
} // end of setCaption()

//! getter method for Font
QFont* QCheckBox::getFont()
{
return Font;
} // end of getFont()

// WARNING : The implementation of this method will be automatically generated with code generator.
// To prevent this add '//KEEP' at the first line of the implementation.(first line after '{')
//! prints itself to the cout
void QCheckBox::print()
{
cout << "Class : QCheckBox" << endl;
cout << "State = " << State << endl;
cout << "BoxSize = " << BoxSize << endl;
cout << "Caption = " << Caption << endl;
} // end of print()

// WARNING : The implementation of this method will be automatically generated with code generator.
// To prevent this add '//KEEP' at the first line of the implementation.(first line after '{')
//! Constructor
 QCheckBox::QCheckBox( QWidget*  aOwner ):QWidget(aOwner)
{
State = cbUnchecked;
BoxSize = 16;
Font = new QFont;
Init();
layout();
} // end of QCheckBox()

// WARNING : The implementation of this method will be automatically generated with code generator.
// To prevent this add '//KEEP' at the first line of the implementation.(first line after '{')
//! Destructor
 QCheckBox::~QCheckBox()
{
delete Font;
CleanUp();
} // end of ~QCheckBox()

// end of implementation of QCheckBox


