// Class QScreen 
// Screen which manages the windows
// Filename QScreen.cpp
// FaPra 2003-2004, A5,  Alexander Kramer


#include "QScreen.h"
#include <iostream>
using namespace std;

// ----------------------------------------------------------

//! draws itself with OpenGL. Here is the OpenGL drawing code
void QScreen::draw( QEvent*  e )
{
  glColor3f(0.3,0.3,0.5);
  glBegin(GL_POLYGON);
  LookAndFeel->drawRect(0,0,Width,Height);
  glEnd();
} // end of draw()

//! layout the children widgets. This method is called every time the size is changed. Have to be overwritten for respond on size changes
void QScreen::layout()
{
} // end of layout()

//! have to return true if widget should be dragged with mouse. This method is overwritten in sub classes. If returns true, widget will be dragged
bool QScreen::canDrag( QEvent*  e )
{
 return false;
} // end of canDrag()

//! processing mouse/keyboard events which are in widgets region. Mouse coordinates are relative to Owner
bool QScreen::processEvent( QEvent*  e )
{
 return false;
} // end of processEvent()

//! processing mouse events which are in Owner widget region. Mouse coordinates are relative to Owner
bool QScreen::processMouseOwner( QEvent*  e )
{
 return false;
} // end of processMouseOwner()

//! called if a child is added/removed
void QScreen::childrenChanged()
{
} // end of childrenChanged()

//! defines whether control can have keyboard focus. Should return true if Widget wants to get keyboard focus
bool QScreen::canFocus( QEvent*  e )
{
 return false;
} // end of canFocus()

//! initialization code
void QScreen::Init()
{
  LookAndFeel = new QLookAndFeel();
} // end of Init()

//! clean up code
void QScreen::CleanUp()
{
  delete LookAndFeel;
} // end of CleanUp()

//! overwritten method which adds the widget to the list for freeing
void QScreen::freeWidget( QWidget*  widget )
{
  // make sure it is not already there
  WidgetsToRemove.remove(widget);

  //add widget
  WidgetsToRemove.push_back(widget);

} // end of freeWidget()

//! removes a widget from the children
void QScreen::remove( QWidget*  widget )
{
  if (FocusPath == widget) this->ScreenFocusedWidget = 0;
  QWidget::remove(widget);
} // end of remove()

//! sends current event
void QScreen::sendEvent()
{
 QWidget* w;
 
 // first remove widgets that are marked for deletion
 while (WidgetsToRemove.size() > 0)
 {
    // get first widget
    w = WidgetsToRemove.front();
    // if widget has a parent tell parent to remove it
    if (w->getOwner()) w->getOwner()->remove(w);
    // delete widget
    delete w;
    // remove the deleted widget from deletion list
    WidgetsToRemove.pop_front();
 }
 
 event(LastEvent);
} // end of sendEvent()

//! post request for redisplay
void QScreen::redraw()
{
  glutPostRedisplay();
} // end of redraw()

//! method for handling glut resize function
void QScreen::glutResize( int  width , int  height )
{
} // end of glutResize()

//! method for handling glut keyboard down function
void QScreen::glutKeyboard( unsigned  char  key , int  x , int  y )
{
  LastEvent->setType(etKeyboard);
  LastEvent->setKey(key);
  LastEvent->setX(x);
  LastEvent->setY(y);
  sendEvent();
  redraw();
} // end of glutKeyboard()

//! method for handling glut keyboard up function 
void QScreen::glutKeyboardUp( unsigned  char  key , int  x , int  y )
{
  LastEvent->setType(etKeyboardUp);
  LastEvent->setKey(key);
  LastEvent->setX(x);
  LastEvent->setY(y);
  sendEvent();
  redraw();
} // end of glutKeyboardUp()

//! method for handling glut special key functions
void QScreen::glutSpecialFunc( int  key , int  x , int  y )
{
  LastEvent->setType(etSpecialKey);
  LastEvent->setKey(key);
  LastEvent->setX(x);
  LastEvent->setY(y);
  sendEvent();
  redraw();
} // end of glutSpecialFunc()

//! method for handling glut mouse motion function
void QScreen::glutMotion( int  x , int  y )
{
  LastEvent->setType(etMouseMotion);
  LastEvent->setX(x);
  LastEvent->setY(y);
  sendEvent();
  redraw();
} // end of glutMotion()

//! method for handling glut mouse button function
void QScreen::glutMouse( int  button , int  state , int  x , int  y )
{
  LastEvent->setX(x);
  LastEvent->setY(y);

  switch (button)
   {
    case GLUT_LEFT_BUTTON: LastEvent->setButton(mbLeft);
    break;
    case GLUT_RIGHT_BUTTON: LastEvent->setButton(mbRight);
    break;
    case GLUT_MIDDLE_BUTTON: LastEvent->setButton(mbCenter);
    break;
   }

  if (state == GLUT_DOWN)
  {
     LastEvent->setType(etMouseDown);
     sendEvent();
  }
  else if (state == GLUT_UP)
  {
     LastEvent->setType(etMouseUp);
     sendEvent();
     LastEvent->setType(etMouseClick);
     sendEvent();
  }
  redraw();
} // end of glutMouse()

//! method for handling glut display function
void QScreen::glutDisplay()
{
  LastEvent->setType(etDraw);
  sendEvent();
} // end of glutDisplay()

//! called if Sender wants to announce new focused control
void QScreen::requestFocus( QWidget*  Sender , QWidget*  FocusedWidget )
{
  if ((FocusPath != 0) && (FocusPath != Sender))
   {
      FocusPath->clearFocusPath();
   }
   
  //if (ScreenFocusedWidget) ScreenFocusedWidget->setFocused(false);
  FocusPath = Sender;
  ScreenFocusedWidget = FocusedWidget;
 // FocusedWidget->setFocused(true);
  redraw();
} // end of requestFocus()

//! getter method for ScreenFocusedWidget
QWidget* QScreen::getScreenFocusedWidget()
{
return ScreenFocusedWidget;
} // end of getScreenFocusedWidget()

//! setter method for ScreenFocusedWidget
void QScreen::setScreenFocusedWidget( QWidget*  newValue )
{
ScreenFocusedWidget = newValue;
} // end of setScreenFocusedWidget()

//! getter method for LastEvent
QEvent* QScreen::getLastEvent()
{
return LastEvent;
} // end of getLastEvent()

// WARNING : The implementation of this method will be automatically generated with code generator.
// To prevent this add '//KEEP' at the first line of the implementation.(first line after '{')
//! prints itself to the cout
void QScreen::print()
{
 //KEEP
cout << "Class : QScreen" << endl;
//cout << "WidgetsToRemove = " << WidgetsToRemove << endl;
cout << "ScreenFocusedWidget = " << ScreenFocusedWidget << endl;
} // end of print()

// WARNING : The implementation of this method will be automatically generated with code generator.
// To prevent this add '//KEEP' at the first line of the implementation.(first line after '{')
//! Constructor
 QScreen::QScreen( QWidget*  aOwner ):QWidget(aOwner)
{
ScreenFocusedWidget = 0;
LastEvent = new QEvent;
Init();
layout();
} // end of QScreen()

// WARNING : The implementation of this method will be automatically generated with code generator.
// To prevent this add '//KEEP' at the first line of the implementation.(first line after '{')
//! Destructor
 QScreen::~QScreen()
{
delete LastEvent;
CleanUp();
} // end of ~QScreen()

// end of implementation of QScreen


