// Class QScreen 
// Filename QScreen.h 
// FaPra 2003-2004, A5, Alexander Kramer

#ifndef QScreen_h 
#define QScreen_h 


//! Pre-declaration of QScreen
class QScreen;


#include "QEvent.h"
#include "QWidget.h"

//! Screen which manages the windows
class QScreen : public QWidget
{
protected:
//! Widget List which will be removed before next event processing
list<QWidget*> WidgetsToRemove;

//! current focused widget
QWidget* ScreenFocusedWidget;

//! last event which was sent to windows or widgets
QEvent* LastEvent;

public:
//! draws itself with OpenGL. Here is the OpenGL drawing code
virtual void draw( QEvent*  e );

//! layout the children widgets. This method is called every time the size is changed. Have to be overwritten for respond on size changes
virtual void layout();

//! have to return true if widget should be dragged with mouse. This method is overwritten in sub classes. If returns true, widget will be dragged
virtual bool canDrag( QEvent*  e );

//! processing mouse/keyboard events which are in widgets region. Mouse coordinates are relative to Owner
virtual bool processEvent( QEvent*  e );

//! processing mouse events which are in Owner widget region. Mouse coordinates are relative to Owner
virtual bool processMouseOwner( QEvent*  e );

//! called if a child is added/removed
virtual void childrenChanged();

//! defines whether control can have keyboard focus. Should return true if Widget wants to get keyboard focus
virtual bool canFocus( QEvent*  e );

//! initialization code
virtual void Init();

//! clean up code
virtual void CleanUp();

//! overwritten method which adds the widget to the list for freeing
virtual void freeWidget( QWidget*  widget );

//! removes a widget from the children
virtual void remove( QWidget*  widget );

//! sends current event
 void sendEvent();

//! post request for redisplay
virtual void redraw();

//! method for handling glut resize function
virtual void glutResize( int  width , int  height );

//! method for handling glut keyboard down function
virtual void glutKeyboard( unsigned  char  key , int  x , int  y );

//! method for handling glut keyboard up function 
virtual void glutKeyboardUp( unsigned  char  key , int  x , int  y );

//! method for handling glut special key functions
virtual void glutSpecialFunc( int  key , int  x , int  y );

//! method for handling glut mouse motion function
virtual void glutMotion( int  x , int  y );

//! method for handling glut mouse button function
virtual void glutMouse( int  button , int  state , int  x , int  y );

//! method for handling glut display function
virtual void glutDisplay();

//! called if Sender wants to announce new focused control
virtual void requestFocus( QWidget*  Sender , QWidget*  FocusedWidget );

//! getter method for ScreenFocusedWidget
virtual QWidget* getScreenFocusedWidget();

//! setter method for ScreenFocusedWidget
virtual void setScreenFocusedWidget( QWidget*  newValue );

//! getter method for LastEvent
 QEvent* getLastEvent();

//! prints itself to the cout
 void print();

//! Constructor
  QScreen( QWidget*  aOwner );

//! Destructor
virtual  ~QScreen();


};
#endif

