// Class QToolBar 
// Tool bar which is movable and layouts its widgets horizontal
// Filename QToolBar.cpp
// FaPra 2003-2004, A5,  Alexander Kramer


#include "QToolBar.h"
#include <iostream>
using namespace std;

// ----------------------------------------------------------

//! brings the given widget to the front. Must be a child widget
void QToolBar::bringToFront( QWidget*  widget )
{
  
} // end of bringToFront()

//! draws itself with OpenGL. Here is the OpenGL drawing code
void QToolBar::draw( QEvent*  e )
{
  // clear region with background color
  LookAndFeel->getBackGroundColor()->setAsCurrent();
  glBegin(GL_POLYGON);
  LookAndFeel->drawRect(0,0,Width,Height);
  glEnd();
  
  LookAndFeel->drawBevel(0,0,Width,Height,bsUp);

  // draw Mover on the left side
  LookAndFeel->drawBevel(2,2,2,Height-4,bsUp);
  LookAndFeel->drawBevel(5,2,2,Height-4,bsUp);

} // end of draw()

//! layout the children widgets. This method is called every time the size is changed. Have to be overwritten for respond on size changes
void QToolBar::layout()
{
  if (Children.size() > 0)
  {
    int WidgetWidth = ((Width - MoverWidth) / Children.size()) - 2;
    int border = 2;
    int WidgetHeight = Height - (border*2);
    int i = 0;

    list<QWidget*>::reverse_iterator iter;

    // Iterate through list
    for (iter=Children.rbegin(); iter != Children.rend(); iter++)
    {
      (*iter)->setTop(border);
      (*iter)->setHeight(WidgetHeight);
      (*iter)->setWidth(WidgetWidth);
      (*iter)->setLeft(MoverWidth+((WidgetWidth+2)*i));
      i++;
    }
  }
} // end of layout()

//! have to return true if widget should be dragged with mouse. This method is overwritten in sub classes. If returns true, widget will be dragged
bool QToolBar::canDrag( QEvent*  e )
{
 return e->getX()-Left < MoverWidth;
} // end of canDrag()

//! processing mouse/keyboard events which are in widgets region. Mouse coordinates are relative to Owner
bool QToolBar::processEvent( QEvent*  e )
{
 return false;
} // end of processEvent()

//! processing mouse events which are in Owner widget region. Mouse coordinates are relative to Owner
bool QToolBar::processMouseOwner( QEvent*  e )
{
 return false;
} // end of processMouseOwner()

//! called if a child is added/removed
void QToolBar::childrenChanged()
{
  layout();
} // end of childrenChanged()

//! defines whether control can have keyboard focus. Should return true if Widget wants to get keyboard focus
bool QToolBar::canFocus( QEvent*  e )
{
 return false;
} // end of canFocus()

//! initialization code
void QToolBar::Init()
{
  Width = 100;
  Height = 25;
} // end of Init()

//! clean up code
void QToolBar::CleanUp()
{
 
} // end of CleanUp()

//! getter method for MoverWidth
int QToolBar::getMoverWidth()
{
return MoverWidth;
} // end of getMoverWidth()

//! setter method for MoverWidth
void QToolBar::setMoverWidth( int  newValue )
{
MoverWidth = newValue;
} // end of setMoverWidth()

// WARNING : The implementation of this method will be automatically generated with code generator.
// To prevent this add '//KEEP' at the first line of the implementation.(first line after '{')
//! prints itself to the cout
void QToolBar::print()
{
cout << "Class : QToolBar" << endl;
cout << "MoverWidth = " << MoverWidth << endl;
} // end of print()

// WARNING : The implementation of this method will be automatically generated with code generator.
// To prevent this add '//KEEP' at the first line of the implementation.(first line after '{')
//! Constructor
 QToolBar::QToolBar( QWidget*  aOwner ):QWidget(aOwner)
{
MoverWidth = 10;
Init();
layout();
} // end of QToolBar()

// WARNING : The implementation of this method will be automatically generated with code generator.
// To prevent this add '//KEEP' at the first line of the implementation.(first line after '{')
//! Destructor
 QToolBar::~QToolBar()
{
CleanUp();
} // end of ~QToolBar()

// end of implementation of QToolBar


