// Class QWindow 
// Filename QWindow.h 
// FaPra 2003-2004, A5, Alexander Kramer

#ifndef QWindow_h 
#define QWindow_h 


//! Pre-declaration of QWindow
class QWindow;


#include "QButton.h"
#include "QColor.h"
#include "QFont.h"
#include "QWidget.h"

//! Widget which is a GUI-Window.
class QWindow : public QWidget
{
private:
//! indicates whether the window is currently resizing
bool Resizing;

//! old position for restore from maximize
int OldLeft;

//! old position for restore from maximize
int OldTop;

//! old position for restore from maximize
int OldWidth;

//! old position for restore from maximize
int OldHeight;

//! position x at the beginning of the resizing
int ResizeStartX;

//! position y at the beginning of the resizing
int ResizeStartY;

protected:
//! titlebar height
int TitleBarHeight;

//! titlebar border
int TitleBarBorder;

//! caption of the window
string Caption;

//! minimum allowed width if resizing with mouse
int MinResizeWidth;

//! minimum allowed height if resizing with mouse
int MinResizeHeight;

//! indicates whether the window is currently maximized
bool Maximized;

//! defines whether the window can be resized by mouse
bool Resizable;

//! Button for close
QButton* CloseButton;

//! Button for maximize
QButton* MaximizeButton;

//! widget for caption text
QFont* Font;

//! color of Title bar
QColor* TitleBarColor;

//! color of title bar if window is not active
QColor* TitleBarColorInactive;

public:
//! draws frame
virtual void drawFrame();

//! draws info about size/position. Semi transparent rectangle with window font 
 void drawCoordInfos( int  x , int  y , const  char*  Text );

//! draws close button
 void drawCloseButton();

//! draws maximize button
 void drawMaximizeButton();

//! maximize / restore window. If window is maximized, restore its size/position, else maximize its size 
 void maximize();

//! draws itself with OpenGL. Here is the OpenGL drawing code
virtual void draw( QEvent*  e );

//! layout the children widgets. This method is called every time the size is changed. Have to be overwritten for respond on size changes
virtual void layout();

//! have to return true if widget should be dragged with mouse. This method is overwritten in sub classes. If returns true, widget will be dragged
virtual bool canDrag( QEvent*  e );

//! processing mouse/keyboard events which are in widgets region. Mouse coordinates are relative to Owner
virtual bool processEvent( QEvent*  e );

//! processing mouse events which are in Owner widget region. Mouse coordinates are relative to Owner
virtual bool processMouseOwner( QEvent*  e );

//! called if a child is added/removed
virtual void childrenChanged();

//! defines whether control can have keyboard focus. Should return true if Widget wants to get keyboard focus
virtual bool canFocus( QEvent*  e );

//! initialization code
virtual void Init();

//! clean up code
virtual void CleanUp();

//! getter method for TitleBarHeight
virtual int getTitleBarHeight();

//! setter method for TitleBarHeight
virtual void setTitleBarHeight( int  newValue );

//! getter method for TitleBarBorder
virtual int getTitleBarBorder();

//! setter method for TitleBarBorder
virtual void setTitleBarBorder( int  newValue );

//! getter method for Caption
virtual string getCaption();

//! setter method for Caption
virtual void setCaption( string  newValue );

//! getter method for MinResizeWidth
virtual int getMinResizeWidth();

//! setter method for MinResizeWidth
virtual void setMinResizeWidth( int  newValue );

//! getter method for MinResizeHeight
virtual int getMinResizeHeight();

//! setter method for MinResizeHeight
virtual void setMinResizeHeight( int  newValue );

//! getter method for Maximized
virtual bool getMaximized();

//! setter method for Maximized
virtual void setMaximized( bool  newValue );

//! getter method for Resizable
virtual bool getResizable();

//! setter method for Resizable
virtual void setResizable( bool  newValue );

//! getter method for CloseButton
virtual QButton* getCloseButton();

//! getter method for MaximizeButton
virtual QButton* getMaximizeButton();

//! getter method for Font
 QFont* getFont();

//! getter method for TitleBarColor
 QColor* getTitleBarColor();

//! getter method for TitleBarColorInactive
 QColor* getTitleBarColorInactive();

//! prints itself to the cout
 void print();

//! Constructor
  QWindow( QWidget*  aOwner );

//! Destructor
virtual  ~QWindow();


};
#endif

